@@ -4,7 +4,7 @@ import numpy as np
 
 class FeatureEngineering_Numerical:
     
-    def __init__(self, method='10p', custom_buckets=None, adaptive = True, model='clf'):
+    def __init__(self, method='10p', custom_buckets=None, adaptive = True, model_type='clf'):
         '''
         Select various binning techniques for bucketing the numerical variables
 
@@ -27,7 +27,7 @@ class FeatureEngineering_Numerical:
             adaptive : bool. Default True
                 If any numerical variable is skewed in nature then choosing adaptive will try to find the 
                 next change in the percentile value and will not create the similar bins
-            model : str. Default 'clf'
+            model_type : str. Default 'clf'
                 'classification' or 'clf' for classification related analysis 
                 'regression' or 'reg' for regression related analysis
 
@@ -40,7 +40,7 @@ class FeatureEngineering_Numerical:
         self.method = method
         self.custom_buckets = custom_buckets
         self.adaptive = adaptive
-        self.model = model
+        self.model_type = model_type
     
     def fit(self, df, variables):
         
@@ -57,6 +57,8 @@ class FeatureEngineering_Numerical:
 
         '''
         #fe_df = df.copy()
+        df = df.copy()
+
         self.param_dict_ ={}
         
         if self.method == '5p':
@@ -101,7 +103,8 @@ class FeatureEngineering_Numerical:
             df : dataframe 
                 dataframe with binned numerical variables
         '''
-        
+        df = df.copy()
+
         if self.method in ('5p', '10p', '20p', '25p', '95p', '98p', 'custom'):
             for var, binner in self.param_dict_.items():
                 df[var] = pd.cut(df[var], bins = binner, include_lowest=True)
@@ -111,12 +114,12 @@ class FeatureEngineering_Numerical:
         return df
 
 class FeatureEngineering_Categorical:
-    def __init__(self, model=None, method='label', drop_last=False, n_frequent=None):
+    def __init__(self, model_type=None, method='label', drop_last=False, n_frequent=None):
         """
         Parameters:
         -----------
-            model : str. default None. 
-                Most of the encoding methods can be used for both classification and regression problems. however only 2 methods require model to be defined as 'classification' or 'clf'
+            model_type : str. default None. 
+                Most of the encoding methods can be used for both classification and regression problems. however only 2 methods require model_type to be defined as 'classification' or 'clf'
             method : str
                 'rare_encoding' or 'rare' : for combining less frequent categories into 'Rare' label first. 
                 'one-hot' or 'onehot' : for one hot encoding
@@ -128,8 +131,8 @@ class FeatureEngineering_Categorical:
                 Target encoding methods
                 'target_ordered' : converts categories into codes but in the descending order of mean target value
                 'target_mean' : converts categories into mean target value
-                'target_prob_ratio' : only for classification models, takes the ratio of target =1 / target =0
-                'target_woe' : only for classification models, calculates the weight of evidence(woe) for each category and replaces the value for that
+                'target_prob_ratio' : only for classification model_types, takes the ratio of target =1 / target =0
+                'target_woe' : only for classification model_types, calculates the weight of evidence(woe) for each category and replaces the value for that
                 
             drop_last : bool. default False
                 if method = 'one-hot' then use this parameter to drop last category 
@@ -141,7 +144,7 @@ class FeatureEngineering_Categorical:
         self.method = method
         self.drop_last = drop_last
         self.n_frequent = n_frequent
-        self.model = model
+        self.model_type = model_type
         self.drop_last = drop_last
         
     def fit(self, df, variables, target=None, rare_tol=5):
@@ -163,6 +166,7 @@ class FeatureEngineering_Categorical:
         self.rare_tol = rare_tol
         self.param_dict_ = {}
         self.variables = variables
+        df = df.copy()
 
         #Convert to 'Object' type
         for var in self.variables:
@@ -212,7 +216,7 @@ class FeatureEngineering_Categorical:
                 s = df.groupby(var)[target].mean()
                 self.param_dict_[var] = {cat:ix for cat, ix in s.items()}
         
-        if (self.model =='classification' or self.model == 'clf'):
+        if (self.model_type =='classification' or self.model_type == 'clf'):
             if self.method =='target_prob_ratio':
                 for var in variables:
                     try:
@@ -224,7 +228,7 @@ class FeatureEngineering_Categorical:
                     except:
                         print(f'Error in the variable : {var}')
                     
-        if (self.model =='classification' or self.model == 'clf'):
+        if (self.model_type =='classification' or self.model_type == 'clf'):
             if self.method =='target_woe':
                 for var in variables:
                     try:
@@ -250,6 +254,8 @@ class FeatureEngineering_Categorical:
             dataframe with all the variables encoded
         '''
         
+        df = df.copy()
+
         # Convert to object type
         for var in self.variables:
             df[var] = df[var].astype('object')
@@ -271,7 +277,7 @@ class FeatureEngineering_Categorical:
             for var, mapper in self.param_dict_.items():
                 df[var] = df[var].map(mapper)
         
-        if (self.model =='classification' or self.model == 'clf'):
+        if (self.model_type =='classification' or self.model_type == 'clf'):
             if self.method in ('target_prob_ratio', 'target_woe'):
                 for var, mapper in self.param_dict_.items():
                     df[var] = df[var].map(mapper)
@@ -290,19 +296,23 @@ class FeatureEngineering_DateTime:
         
         Parameters:
         -----------
-            df : dataframe
-            datetime_var : 
-                only single datetime variable at one time
-            prefix : prefix for the new variables that are going to get created 
-            drop_orig : If True then original variable is dropped. Usually it's kept as True
+            df : DataFrame
+            drop_orig : bool, default True
+                If True then original variable is dropped
             
         Returns:
         ---------
-            df : returns a new dataframe with more engineered variables
+            df : DataFrame
+                Returns a new dataframe with more engineered variables
             
         Example:
         --------
-            train_new = FeatureEngineering_DateTime (df=train, datetime_var='Saledate', prefix='Saledate_', drop_orig=True )
+            dt_imputer = FeatureEngineering_DateTime (drop_orig=True)
+            dt_imputer.fit(train, ['saledate'], prefix = 'saledate_')
+
+            train = dt_imputer.transform(train)
+
+
         """
         self.drop_orig=drop_orig
 
@@ -319,6 +329,8 @@ class FeatureEngineering_DateTime:
         -----------
 
         '''
+        df = df.copy()
+
         self.datetime_variables = datetime_variables
         self.prefix = prefix
         self.param_dict_ = "No Parameter dictionary needed for this. Use transform() method"
@@ -338,6 +350,7 @@ class FeatureEngineering_DateTime:
                 Transformed dataframe with all the additional features
 
         '''
+        df = df.copy()
 
         for var in self.datetime_variables:
 
