@@ -3,8 +3,10 @@ import numpy as np
 
 from IPython.display import Markdown, display
 
-
-
+'''
+#@ Another way of doing the missing data imputation is to see - 
+event rate of missing data is matching with which category and do imputation with that category
+'''
 
 class MissingDataImputer_Numerical:
 
@@ -146,7 +148,7 @@ class MissingDataImputer_Categorical:
                             0 if there's no missing value in var
         """
         self.method = method
-        self.value = value
+        self.value = str(value)
         self.random_state = random_state
         self.add_indicator = add_indicator
         
@@ -162,8 +164,11 @@ class MissingDataImputer_Categorical:
 
         '''
         self.variables = variables
+
+        for var in variables:
+            df[var] = df[var].astype('object')
         
-        if self.method =='frequent':
+        if self.method in ('frequent', 'mode'):
             self.param_dict_ = {}
             
             for var in variables:
@@ -208,9 +213,14 @@ class MissingDataImputer_Categorical:
         
         else:
             for var in self.param_dict_:
+
+                #convert to object type variable
+                df[var] = df[var].astype('object')
+
                 # Add indicator
                 if self.add_indicator == True:
                     df[var + '_nan'] = np.where(df[var].isnull(), 1, 0)
+                
                 # impute missing values
                 df[var].fillna(self.param_dict_[var] , inplace=True)
         
@@ -220,6 +230,9 @@ class MissingDataImputer_Categorical:
     def __random_imputer__(self, df):
         for var in self.variables:
             
+            #convert to object type variable
+            df[var] = df[var].astype('object')
+
             if df[var].isnull().sum()>0:
                 
                 # number of data point to extract at random
